#include "Single_Vertex.H"
//#include "Message.H"
#include "Vector.H"
#include "String_Tree.H"
#include "Shell_Tools.H"
#include <iomanip>
#include <stdlib.h>
#include <stdio.h>


using namespace AMEGIC;
using namespace ATOOLS;
using namespace std;

// Constructors and Destructors
Single_Vertex::Single_Vertex() : Color(NULL), Lorentz(NULL) 
{ ncf = nlf = t = 0; nleg=3; }

Single_Vertex::Single_Vertex(const Single_Vertex& v): Color(NULL), 
  Lorentz(NULL) { *this=v; }

Single_Vertex::~Single_Vertex(){
      if (Color)   if (ncf==1) delete Color;
      else delete [] Color;
      if (Lorentz) if (nlf==1) delete Lorentz;
      else delete [] Lorentz;
    }


// Operators
Single_Vertex& Single_Vertex::operator=(const Single_Vertex& v) {
      if (Color)   if (ncf==1) delete Color;
                   else        delete [] Color;
      if (Lorentz) if (nlf==1) delete Lorentz;
                   else        delete [] Lorentz;
    
      if (this!=&v) {
	for (short int i=0;i<4;i++) in[i]  = v.in[i];
	for (short int j=0;j<4;j++) cpl[j] = v.cpl[j];
	
	nleg = v.nleg;
	Str  = v.Str;
	on   = v.on;
	ncf  = v.ncf;
	nlf  = v.nlf;
	t=v.t;
	if (ncf==1) Color   = new Color_Function(*v.Color);
	else {
	  Color = new Color_Function[ncf];
	  for (int i=0;i<ncf;i++)
	    Color[i] = v.Color[i];
	}
	if (nlf==1) Lorentz = new Lorentz_Function(*v.Lorentz);
	else {
	  Lorentz = new Lorentz_Function[nlf];
	  for (int i=0;i<nlf;i++)
	    Lorentz[i] = v.Lorentz[i];
	}
      }
      return *this;
    }


 
const bool Single_Vertex::operator==(const Single_Vertex& probe) 
{
  switch (nleg) // different checks for 3-leg and 4-leg vertices
  {
    case 4: return (probe.nleg == 4) &&
       	           (in[0] == probe.in[0]) &&
                   (in[1] == probe.in[1]) &&
                   (in[2] == probe.in[2]) &&
                   (in[3] == probe.in[3]);
    case 3:  return (probe.nleg == 3) &&
	            (in[0] == probe.in[0]) &&
                    (in[1] == probe.in[1]) &&
                    (in[2] == probe.in[2]);
    default: return 0; 
  }
}


ostream& AMEGIC::operator<<(ostream& s, const Single_Vertex& sv)
{
  return s<<'('<<sv.in[0]<<','<<sv.in[1]<<','<<sv.in[2]<<','<<sv.in[3]
          <<") with cpl["<<sv.cpl[0]<<','<<sv.cpl[1]<<','<<sv.cpl[2]<<','<<sv.cpl[3]<<']'
          <<" is "<<((sv.on) ? "on" : "off");
}


// MPI stuff
void AMEGIC::Single_Vertex2MPI(const Single_Vertex * v , MPI_Single_Vertex & mpi_v) {
  
    
  if (!v) {
    for (int i=0;i<4;++i)
      mpi_v.m_fl[i] = 0;
    
    return; 
  }
  /*
  Lorentz_Function2MPI(v->Lorentz,mpi_v.m_lf);
  Color_Function2MPI(v->Color,mpi_v.m_cf);
  */

  for (int i=0;i<4;++i)
    mpi_v.m_fl[i] = int(v->in[i]);

  /*  
  for (int i=0;i<7;i+=2) {
    mpi_v.m_cpl[i]   = real(v->cpl[i/2]);
    mpi_v.m_cpl[i+1] = imag(v->cpl[i/2]);
  }
  */
}


Single_Vertex * AMEGIC::MPI2Single_Vertex(const MPI_Single_Vertex & mpi_v ) {

  Single_Vertex * v ;
  
  v = new Single_Vertex();
  
  for (int i=0;i<4;++i) {
    v->in[i] = Flavour((kf::code)(abs(mpi_v.m_fl[i])));
    if (mpi_v.m_fl[i]<0) v->in[i]=v->in[i].Bar();
  }

  /*
  v->Lorentz = AMEGIC::MPI2Lorentz_Function(mpi_v.m_lf);
  v->Color   = AMEGIC::MPI2Color_Function(mpi_v.m_cf);
  
  for (int i=0;i<7;i+=2) {
    (v->cpl[i/2]) = Complex(mpi_v.m_cpl[i],mpi_v.m_cpl[i+1]);
  }
  */
  
  return v;

}

std::ostream & AMEGIC::operator<<(std::ostream & s, const MPI_Single_Vertex & sv) {
  s<<sv.m_fl[0]<<","<<sv.m_fl[1]<<","<<sv.m_fl[2]<<","<<sv.m_fl[3];
  return s;
}
